#
# header/mnemonic.tcl
#####################
#

proc check_header_mnemonic {} {
	global MAINTYPE
	global SPECHEADER
	global MNEMESCAPE
	if {[is_set MAINTYPE]} {
		if {$MAINTYPE == "MIME"} {
			return 0
		}
	}
	if {[set MNEMCHARSET [getheader "X-Charset"]] != 0} {
		setfromcharset $MNEMCHARSET
		addspecheader X-Charset
		if {[set MNEMESCAPE [getheader "X-Char-Esc"]] != 0} {
			setfromesc $MNEMESCAPE
			addspecheader X-Char-Esc
		}
	}
}

check_header_mnemonic
