#
# convert/sunet.tcl
#
# Default template for SUNET, i.e. iso646-se and uuencoded binaries.

set PART_NUMBER 0
topmessnode
while {1 == 1} {
	rmspecheader
	set MTYPE [gettype]
	case $MTYPE in {
	
	TEXT	{
			do_decode
			tocharset  "ISO-8859-1"
			to7bit
		}

	BINHEX	{
			set JUNK "BINHEX"			
		}

	MULTIPART {
			set JUNK "MULTIPART"
		}

	default	{
			do_code "UUENCODE"
		}

	}
	set A [nextmessnode]
	if {$A < 1} {
		break
	}
	set PART_NUMBER [expr $PART_NUMBER+1]
}
