# run.tcl
#
# This file is executed for each message after collecting information
# about the format of the message. This is where conversions and 
# header changes are set up.
#
#####################################################################
#
# These files will be sourced.
#
# Add any file that should be initially sourced into the SOURCES list.
#
set SOURCES {
check.tcl
}

#
# Source 'em
#
foreach SOURCEFILE $SOURCES {
	if [file exists $appLibrary/check/$SOURCEFILE] {
		source $appLibrary/check/$SOURCEFILE
	} \
	else \
	{
		debug "Unable to source $appLibrary/check/$SOURCEFILE"
	}
}



