#
# convert/eudora.tcl
#
# Default template for Eudora (MacIntosh)
topmessnode
while {1 == 1} {
	joinextension
	rmspecheader
	set MTYPE [gettype]
	set ENC [getencoding]
	case $MTYPE in {
	
	TEXT		{
				do_decode
				tocharset "MACINTOSH"
			}
	
	APPLEDOUBLE	{
				do_code "UUENCODE"
				set APPLEDOUBLE "TRUE"
			}

	APPLESINGLE	{
				do_code "UUENCODE"
			}

	default		{
				if {[is_set APPLEDOUBLE]} {
					do_code "UUENCODE"
					unset APPLEDOUBLE
				} \
				else \
				{
					if {$MTYPE != "MULTIPART"} {
						do_code "BINHEX"
					}
				}
			}
	}
	set A [nextmessnode]
	if {$A < 1} {
		break
	}
}

