/*
 *
 * Copyright (C) 1993 Swedish University Network (SUNET)
 *
 *
 * This program is developed by UDAC, Uppsala University by commission
 * of the Swedish University Network (SUNET). 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITTNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *                                          Martin.Wendel@udac.uu.se
 *
 *                                          Martin Wendel
 *                                          UDAC	
 *                                          Box 174
 *                                          S-751 04 Uppsala
 *                                          Sweden
 */




#ifndef EMIL
#ifndef SENDMAIL
# ifdef _DEFINE
#  define EXTERN
#  ifndef lint
static char MkSccsId[] =      "@(#)mk.h 0.1                12/2/93";
static char MkRcsId[] =       "@(#)$Id: mk.h,v 0.1         1993/02/12";
#  endif /* lint */
# else /* !_DEFINE */
#  define EXTERN extern
# endif /* _DEFINE */

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <syslog.h>
#include "charset/iso646.h"
#include "charset/charset.h"

#ifdef BSD
#include <strings.h>
#else /* !BSD */
#include <string.h>
#endif /* BSD */

#if defined(__STDC__)
# include <stdarg.h>
#else
# include <varargs.h>
#endif /* __STDC__ */

#define sfgets fgets
#define syserr printf
typedef char MAILER;


#include "cdefs.h"
# define newstr(s)  strcpy((char *)xalloc((int)strlen(s) + 1), s);
#endif /* !SENDMAIL */

#endif /* !EMIL */
#ifdef SENDMAIL
#include "sendmail.h"
#define HAS_XALLOC
#endif
#ifndef EMIL
#include "mkconf.h"
#endif
#include "mkspec.h"





