#
# header/sunet-spec.tcl
#####################
#
proc check_sunet_passthru {} {
	if {[is_root]} {
		if {[set SUBJ [getheader "Subject"]] != 0} {
			if {[llength $SUBJ] != 0} {
				set FSUBJ [lindex $SUBJ 0]
				if {[string index $FSUBJ 0] == ":"} {
debug "sunet-spec: Subject starting with colon: dotunnel"
					dotunnel
				}
			}
		}
	}
}
check_sunet_passthru


			
