#
# header/init.tcl
#################
#
if {[is_root]} {
	set CONTCONV [getheader "Content-Conversion"]
	if {[is_set CONTCONV]} {
		set CONTC [string toupper $CONTCONV]
		if {$CONTC == "PROHIBITED"} {
			dotunnel
			return
		}
	}
}
setcheck "uuencode"
setcheck "binhex"




